/*********************************************************************
*
* Copyright 2016-2019 Broadcom.
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
**********************************************************************
*
* @filename  openapi_transfer.h
*
* @purpose   Transfer Download
*
* @component OpEN
*
* @create    2/18/2016
*
* @end
*
**********************************************************************/
#ifndef OPENAPI_TRANSFER_H_INCLUDED
#define OPENAPI_TRANSFER_H_INCLUDED

#include "openapi_common.h"

/** OPEN File Transfer Types */
typedef enum
{
  OPEN_TRANSFER_NONE = 0,     /**< Invalid xfer type   */
  OPEN_TRANSFER_TFTP = 1,     /**< Transfer via tftp   */
  OPEN_TRANSFER_SFTP = 2,     /**< Transfer via sftp   */
  OPEN_TRANSFER_SCP  = 3,     /**< Transfer via scp    */
  OPEN_TRANSFER_HTTP = 4,     /**< Transfer via http   */
  OPEN_TRANSFER_HTTPS = 5,    /**< Transfer via https  */
  OPEN_TRANSFER_LAST = 6      /**< For bounds checking */
} OPEN_TRANSFER_MODES_t;

/**  OPEN File Types         */
typedef enum
{
  OPEN_FILE_TYPE_NONE           = 0,  /**< Invalid file type     */
  OPEN_FILE_TYPE_CODE           = 1,  /**< Compressed executable */
  OPEN_FILE_TYPE_CONFIG         = 2,  /**< Configuration file    */
  OPEN_FILE_TYPE_BACKUP_CONFIG  = 3,  /**< Backup Configuration file */
  OPEN_FILE_TYPE_LAST           = 4   /**< For bounds checking       */
} OPEN_FILE_TYPES_t;

/** OPEN File Transfer Status */
typedef enum
{
  OPEN_NO_STATUS_CHANGE                   = 1,
  OPEN_STATUS_CHANGED                     = 2,
  OPEN_TRANSFER_FAILED                    = 3,
  OPEN_TRANSFER_BAD_CRC                   = 4,
  OPEN_FLASH_FAILED                       = 5,
  OPEN_OPR_FILE_TYPE_FAILED               = 6,
  OPEN_OTHER_FILE_TYPE_FAILED             = 7,
  OPEN_NEXT_DATA_BLOCK_RECEIVED           = 8,
  OPEN_TRANSFER_FILE_NOT_FOUND            = 9,
  OPEN_TRANSFER_SUCCESS                   = 10,
  OPEN_TFTP_FAILED                        = 11,
  OPEN_OPR_FILE_VERSION_FAILED            = 12,  
  OPEN_DIGITAL_SIGNATURE_VERIFY_FAILED    = 13,  
  OPEN_DIGITAL_SIGNATURE_TRUNCATE_FAILED  = 14,
  OPEN_TRANSFER_STATUS_LAST               = 15
} OPEN_TRANSFER_STATUS_t;

/** OPEN File Transfer Code */
typedef enum
{
  OPEN_TRANSFER_CODE_NONE                     = 0,
  OPEN_TRANSFER_CODE_STARTING                 = 1,
  OPEN_TRANSFER_CODE_ERROR_STARTING           = 2,
  OPEN_TRANSFER_CODE_WRONG_FILE_TYPE          = 3,
  OPEN_TRANSFER_CODE_UPDATE_CONFIG            = 4,
  OPEN_TRANSFER_CODE_INVALID_CONFIG           = 5,
  OPEN_TRANSFER_CODE_WRITING_FLASH            = 6,
  OPEN_TRANSFER_CODE_FLASH_FAILED             = 7,
  OPEN_TRANSFER_CODE_CHECKING_CRC             = 8,
  OPEN_TRANSFER_CODE_CRC_FAILURE              = 9,
  OPEN_TRANSFER_CODE_UNKNOWN_DIRECTION        = 10,
  OPEN_TRANSFER_CODE_SUCCESS                  = 11,
  OPEN_TRANSFER_CODE_FAILURE                  = 12,
  OPEN_TRANSFER_CODE_WRONG_DEVICE             = 13,
  OPEN_TRANSFER_CODE_INSUFFICIENT_MEMORY      = 14, 
  OPEN_TRANSFER_CODE_FLASH_FILE_OPEN_FAIL     = 15,
  OPEN_TRANSFER_CODE_ERROR_READING_FROM_FLASH = 16,
  OPEN_TRANSFER_CODE_FILE_DOES_NOT_EXISTS     = 17,
  OPEN_TRANSFER_CODE_FILE_TOO_BIG             = 18,
  OPEN_TRANSFER_CODE_ERROR_WRITING_TO_FLASH   = 19,
  OPEN_TRANSFER_CODE_OVERWRITE_DENIED         = 20,
  OPEN_TRANSFER_CODE_LAST                     = 21
} OPEN_TRANSFER_CODE_t;

/** OPEN Image Verification Options */
typedef enum
{
  OPEN_DIGITAL_SIGNATURE_VERIFY_NONE               = 0, /**< No Digital Signature verify for all downloads */
  OPEN_DIGITAL_SIGNATURE_VERIFY_IMAGE_ONLY         = 1, /**< Digital Signature verify for Image download only */
  OPEN_DIGITAL_SIGNATURE_VERIFY_CONFIG_SCRIPT_ONLY = 2, /**< Digital Signature verify for config script download only */
  OPEN_DIGITAL_SIGNATURE_VERIFY_ALL                = 3, /**< Digital Signature verify for all downloads */
  OPEN_DIGITAL_SIGNATURE_VERIFY_LAST               = 4  /**< For bounds checking   */
} OPEN_DIGITAL_SIGNATURE_OPTION_t;

/** OPEN Configuration File Copy Combinations */
typedef enum
{
  OPEN_COPY_CONFIG_DIRECTIVE_RTOF = 1, /**< "running-config" to "factory-default" */
  OPEN_COPY_CONFIG_DIRECTIVE_RTOS = 2, /**< "running-config" to "startup-config" */
  OPEN_COPY_CONFIG_DIRECTIVE_STOB = 3, /**< "startup-config" to "backup-config" */
  OPEN_COPY_CONFIG_DIRECTIVE_BTOS = 4  /**< "backup-config"  to "startup-config" */
} OPEN_COPY_CONFIG_DIRECTIVE_t;

/*********************************************************************
* @purpose  Determine the size the image file header for this device.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    headerSize      @b{(output)} Size of the image file's header.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @end
*********************************************************************/
open_error_t openapiImageHeaderSizeGet(openapiClientHandle_t *client_handle, 
                                       uint32_t *headerSize);

/*********************************************************************
* @purpose  Test an image file header to determine if it is appropriate for
*           this device.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    headerData      @b{(input)}  Buffer containing the file header from
*                                        the beginning of the file. The size of
*                                        which is determined by calling
*                                        openapiImageHeaderSizeGet().
* @param    isValid         @b{(output)} @ref OPEN_TRUE indicates the code 
*                                        image is intended or suitable for this
*                                        system, else @ref OPEN_FALSE
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    The caller is responsible for sending the number of bytes 
*           directly from the top of the code image file.
* @notes    Buffer containing the file header should have no more than
*           @ref OPENAPI_STK_HEADER_SIZE bytes.
*
* @end
*********************************************************************/
open_error_t openapiImageHeaderCheck(openapiClientHandle_t *client_handle, 
                                     open_buffdesc *headerData,
                                     OPEN_BOOL_t *isValid);

/*********************************************************************
* @purpose  Get the current transfer mode
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    xfrMode         @b{(output)} Currently set transfer type. 
*                                        @ref OPEN_TRANSFER_MODES_t
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferModeGet(openapiClientHandle_t *client_handle, 
                                    OPEN_TRANSFER_MODES_t *xfrMode);

/*********************************************************************
* @purpose  Set the current transfer mode
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    xfrMode         @b{(input)}  On success, current transfer type.
*                                        @ref OPEN_TRANSFER_MODES_t
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferModeSet(openapiClientHandle_t *client_handle, 
                                    OPEN_TRANSFER_MODES_t xfrMode);

/*********************************************************************
* @purpose  Get the Transfer File Type
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    fileType        @b{(output)} Current file transfer type. 
*                                        @ref OPEN_FILE_TYPES_t
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    Only @ref OPEN_FILE_TYPE_CODE is supported by OpEN API for this
*           release.
*
* @end
*********************************************************************/
open_error_t openapiTransferFileTypeGet(openapiClientHandle_t *client_handle, 
                                        OPEN_FILE_TYPES_t *fileType);

/*********************************************************************
* @purpose  Set the Transfer File Type
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    fileType        @b{(input)}  On success, transfer file type. 
*                                        @ref OPEN_FILE_TYPES_t
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    Only @ref OPEN_FILE_TYPE_CODE is supported by OpEN API for this release.
*
* @end
*********************************************************************/
open_error_t openapiTransferFileTypeSet(openapiClientHandle_t *client_handle, 
                                        OPEN_FILE_TYPES_t fileType);

/*********************************************************************
* @purpose  Get the transfer server address.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    inetAddr        @b{(output)} IPv4 or IPv6 address in network byte
*                                        order of the server.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferServerAddressGet(openapiClientHandle_t *client_handle,
                                             open_inet_addr_t *inetAddr);

/*********************************************************************
* @purpose  Set the transfer server address.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    inetAddr        @b{(input)}  IPv4 or IPv6 address in network byte
*                                        order of the server.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    The inet address used for this API can come from the inetAddr value
*           output by the openapiInetAddrFromIPAddrHostNameStr() API. 
* 
* @end
*********************************************************************/
open_error_t openapiTransferServerAddressSet(openapiClientHandle_t *client_handle,
                                             open_inet_addr_t inetAddr);

/*********************************************************************
* @purpose  Get the path and name of the file on the remote system that is
*           designated for transfer.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    filePath        @b{(output)} Path to the file on the remote system
* @param    fileName        @b{(output)} Name of the file on the remote system
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @notes    Caller must provide a buffer for filePath of at least
*           @ref OPENAPI_FILEPATH_LEN_MAX bytes.
* @notes    Caller must provide a buffer for fileName of at least
*           @ref OPENAPI_FILENAME_LEN_MAX bytes.
* 
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferFileRemoteGet(openapiClientHandle_t *client_handle,
                                          open_buffdesc *filePath, 
                                          open_buffdesc *fileName);

/*********************************************************************
* @purpose  Set the path and name of the file on the remote system that is
*           designated for transfer.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    filePath        @b{(input)}  Path to the file on the remote system
* @param    fileName        @b{(input)}  Name of the file on the remote system
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @notes    The filePath should be no longer that @ref OPENAPI_FILEPATH_LEN_MAX
*           bytes
* @notes    The fileName should be no longer that @ref OPENAPI_FILENAME_LEN_MAX
*           bytes
* 
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferFileRemoteSet(openapiClientHandle_t *client_handle,
                                          open_buffdesc *filePath, 
                                          open_buffdesc *fileName);

/*********************************************************************
* @purpose  Get the label by which the file to be transferred is known by on
*           the local system.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    imageLabel      @b{(output)} Label of the file on the local system
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    This API provides the identifying string (label) used to refer to 
*           the file residing in non-volatile memory on the switch, such as
*           "active" or "backup".
* @notes    Buffer size for image labels should be openapiImageFileNameMaxLengthGet().
*
* @end
*********************************************************************/
open_error_t openapiTransferFileLocalGet(openapiClientHandle_t *client_handle, 
                                         open_buffdesc *imageLabel);

/*********************************************************************
* @purpose  Set the label by which the file to be transferred is known by on
*           the local system.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    imageLabel      @b{(input)}  Label of the file on the local system
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    This API sets the identifying string (label) used to refer to the
*           file residing in non-volatile memory on the switch, such as "active"
*           or "backup".
* @notes    The imageLabel should be no longer that openapiImageFileNameMaxLengthGet().
*           bytes
*
* @end
*********************************************************************/
open_error_t openapiTransferFileLocalSet(openapiClientHandle_t *client_handle, 
                                         open_buffdesc *imageLabel);

/*********************************************************************
* @purpose  Get the credentials (username and password) of remote host that
*           will serve the file during the transfer operation.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    username        @b{(output)} Remote host username
* @param    password        @b{(output)} Remote host password
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    This is used with SCP and SFTP file transfers only
* @notes    The buffer for username should be at least
*           @ref OPENAPI_TRANSFER_USERNAME_LEN_MAX bytes.
* @notes    The buffer password should be at least
*           @ref OPENAPI_TRANSFER_PASSWORD_LEN_MAX bytes.
*
* @end
*********************************************************************/
open_error_t openapiTransferRemoteUserCredentialsGet(openapiClientHandle_t *client_handle, 
                                                     open_buffdesc *username, 
                                                     open_buffdesc *password);

/*********************************************************************
* @purpose  Set the credentials (username and password) of remote host that
*           will serve the file during the transfer operation.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    username        @b{(input)}  Remote host username
* @param    password        @b{(input)}  Remote host password
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    This is used with SCP and SFTP file transfers only
* @notes    The username should be no longer than
*           @ref OPENAPI_TRANSFER_USERNAME_LEN_MAX bytes.
* @notes    The password should be no longer than
*           @ref OPENAPI_TRANSFER_PASSWORD_LEN_MAX bytes.
*
* @end
*********************************************************************/
open_error_t openapiTransferRemoteUserCredentialsSet(openapiClientHandle_t *client_handle, 
                                                     open_buffdesc *username, 
                                                     open_buffdesc *password);

/*********************************************************************
* @purpose  Completes an HTTP(S) file download transfer operation. 
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    filePathBuf     @b{(input)}  The complete path to the downloaded 
*                                        file written into a temporary location 
*                                        by the webserver
* @param    fileNameBuf     @b{(input)}  The original source filename. Used 
*                                        when preserving its name.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.20
* 
* @notes    Caller is responsible to check transfer progress and final result 
*           using openapiTransferInProgressGet() and openapiTransferResultGet().
*
* @notes    This function will block management port access until complete.
*
* @notes    This function must only be called after a webserver on the switch
*           has finished the file transfer and the data is located in the file 
*           specified by filePathBuf. 
*
* @end
*********************************************************************/
open_error_t openapiHttpTransferDownStartSet(openapiClientHandle_t *client_handle,
                                             open_buffdesc *filePathBuf, 
                                             open_buffdesc *fileNameBuf);

/*********************************************************************
* @purpose  Starts an asynchronous file download transfer operation. 
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
* 
* @notes    Caller is responsible to check transfer progress and final result 
*           using openapiTransferInProgressGet() and openapiTransferResultGet().
*
* @end
*********************************************************************/
open_error_t openapiTransferDownStartSet(openapiClientHandle_t *client_handle);

/*********************************************************************
* @purpose  Retrieve the progress of a file transfer
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    isInProgress    @b{(output)} @ref OPEN_TRUE indicates a code
*                                        download is in progress.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferInProgressGet(openapiClientHandle_t *client_handle, 
                                          OPEN_BOOL_t *isInProgress);

/*********************************************************************
* @purpose  To retrieve the status of a transfer task in string format
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    resultCode      @b{(output)} Enumerated value indicating the
*                                        transfer download result
* @param    resultStatus    @b{(output)} String value describing @ref resultCode
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @notes    The buffer for resultStatus should be at least
*           @ref OPENAPI_TRANSFER_RESULT_LEN_MAX bytes.
* 
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferResultGet(openapiClientHandle_t *client_handle, 
                                      OPEN_TRANSFER_STATUS_t *resultCode,
                                      open_buffdesc *resultStatus);

/*********************************************************************
* @purpose  To retrieve the status of a transfer operation
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    resultCode      @b{(output)} Enumerated value indicating the
*                                        transfer download result code
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiTransferDownloadResultCodeGet(openapiClientHandle_t *client_handle, 
                                                  OPEN_TRANSFER_CODE_t *resultCode);

/*********************************************************************
* @purpose  To get the option to verify digital signature of downloaded file.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    dsvConfigOption @b{(output)} Enumerated value indicating the digital
*                                        signature verify option.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiDigitalSignatureVerifyConfigGet(openapiClientHandle_t *client_handle,
                                                    OPEN_DIGITAL_SIGNATURE_OPTION_t *dsvConfigOption); 

/*********************************************************************
* @purpose  To set the option to verify digital signature of downloaded file.
*
* @param    client_handle   @b{(input)}  Client handle from registration API.
* @param    dsvConfigOption @b{(input)}  Enumerated value indicating the digital
*                                        signature verify option.
* 
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.10
*
* @end
*********************************************************************/
open_error_t openapiDigitalSignatureVerifyConfigSet(openapiClientHandle_t *client_handle,
                                                    OPEN_DIGITAL_SIGNATURE_OPTION_t dsvConfigOption); 

/*********************************************************************
* @purpose  This API copies configuration files that are located on the
*           switch. The source and destination files as well as their
*           direction is limited to those defined in
*           OPEN_COPY_CONFIG_DIRECTIVE_t.
*
* @param    client_handle @b{(input)} Client handle from registration API.
* @param    directive     @b{(input)} Source to destination definition.
*
* @returns  OPEN_E_NONE      if validation is successful.
* @returns  OPEN_E_PARAM     if parameters are not defined correctly.
* @returns  OPEN_E_UNAVAIL   Not supported for this platform.
*
* @supported-in-version OpEN API Version: 1.20
*
* @end
*********************************************************************/
open_error_t openapiCopyConfigFile(openapiClientHandle_t *client_handle,
                                   OPEN_COPY_CONFIG_DIRECTIVE_t directive);

#endif /* OPENAPI_TRANSFER_H_INCLUDED */
