#!/usr/bin/python

#*********************************************************************
#
# Copyright 2016 Broadcom.
#
#  Licensed under the Apache License, Version 2.0 (the "License")
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#***********************************************************************

# Limitations:
#     - Doesn't support multicast other than "unknown-dst"

import subprocess
import sys
import socket
import fcntl
import time
import struct
from vtepc_libs.gateway_config import Gateway, Device
from vtepc_libs.tenant_config import Tenant

import argparse

sys.path=['/etc/vtepc'] + sys.path

from novaclient.v1_1 import client as nova_client
from keystoneclient.v2_0 import client as keystone_client
from neutronclient.v2_0 import client as neutron_client

import ovs.dirs
import ovs.util
import ovs.daemon
import ovs.unixctl.server
import ovs.vlog

import vtepc_config

class LogicalSwitch():

    '''
    Abstraction of a logical switch. 
    '''

    def __init__(self):
        self._nodes = []
        self._tenant_name = None
        self._network_name = None
        self._segmentation_id = None

    def set_tenant_name(self, name):
        self._tenant_name = name
       
    def get_tenant_name(self):
        return self._tenant_name

    def set_network_name(self, name):
        self._network_name = name
       
    def get_network_name(self):
        return self._network_name

    def get_name(self):
        return "broadcom_openstack_%s_%s" % (self._tenant_name, self._network_name)

    def set_segmentation_id(self, segmentation_id):
        self._segmentation_id = segmentation_id
       
    def get_segmentation_id(self):
        return self._segmentation_id

    def is_node(self, name):

        '''
        If a node with the given name is in the list, return True
        '''

        ret = False
        for x in self._nodes:
            if x == name:
                ret = True
                break
        return ret

    def add_node(self, name):
        if self._is_node(name) == False:
            self._nodes.append(name)

    def del_node(self, name):
        try:
            self._nodes.remove(name) 
        except:
            pass

class GatewayController():

    '''
    Abstration of the controller. Intended to be used as a singleton,
    this class maintains the configuration of the VTEP side cluster.
    '''

    def __init__(self):
        self._devices = []
        self._tenants = []
        self._gateways = []
        self._tenant_to_device_map = []
        self._tenant_to_gateway_map = []
        self._vlan = 2
        self._logical_switches = []


    def get_tenant(self, name):
        ret = None
        for x in self._tenants:
          if x.get_name() == name:
            ret = x
            break;
        return ret 

    def get_gateway(self, ip):
        ret = None
        for x in self._gateways:
          if x.get_ip() == ip:
            ret = x
            break;
        return ret 

    def get_device(self, name):
        ret = None
        for x in self._devices:
          if x.get_name() == name:
            ret = x
            break;
        return ret

    def map_device_to_tenant(self, tenant_name, device_name):

        '''
        Add an entry to the tenant to device map. Items in this map
        indicate that a given tenant wants access to a a given device.
        '''

        for x in self._tenant_to_device_map:
            if x['tenant'] == tenant_name and x['device'] == device_name:
                return

        x = {'tenant': tenant_name, 'device': device_name}
        self._tenant_to_device_map.append(x)

    def get_tenant_device_map(self, tenant):

        '''
        Return the items in the tenant device map for a given tenant
        '''

        ret = []
        for x in self._tenant_to_device_map:
            if x["tenant"] == tenant:
                ret.append(x)
        return ret

    def is_tenant_device(self, tenant, dev):

        '''
        Return True if the tenant and device are in the tenant to device
        map.
        ''' 
        
        ret = False
        for x in self._tenant_to_device_map:
            if x["tenant"] == tenant and x["device"] == dev:
                ret = True
                break
        return ret

    def map_gateway_to_tenant(self, tenant_name, gateway):
        '''
        Add an entry to the tenant to gateway device map. Items in this
        map indicate that a given tenant has one or more devices on the
        switch that it would like access to.
        '''

        for x in self._tenant_to_gateway_map:
            if x['tenant'] == tenant_name and x['gateway'] == gateway:
                return

        x = {'tenant': tenant_name, 'gateway': gateway}
        self._tenant_to_gateway_map.append(x)

    def get_tenant_gateway_map(self, tenant):

        '''
        Return a list of items from the tenant to gateway map for the
        specified tenant.
        '''

        ret = []
        for x in self._tenant_to_gateway_map:
            if x["tenant"] == tenant:
                ret.append(x)
        return ret

    def add_device(self, device):
        '''
        Add a device to the list known by the controller
        '''

        self._devices.append(device)


    def add_gateway(self, gateway):
        '''
        Add a VTEP switch/gateway to the list of configured switches.
        '''

        self._gateways.append(gateway)

    def get_gateway_for_device(self, devicename):
        '''
        Give the name of a device, return the VTEP gateway it can 
        be found on.
        '''

        ret = None
        for x in self._gateways:
            y = x.get_device_by_name(devicename)
            if y != None:
                ret = x
                break
        return ret

    def search(self, name):

        '''
        Find the given tenant object by name. Return None of not found.
        '''

        ret = None
        for x in self._tenants:
            if x.get_name() == name:
                ret = x
                break
        return ret
            
    def add_tenant(self, tenant):

        ''' 
        Add the specified tenant to the list maintained by the controller.
        '''

        if self.search(tenant) == None:
            self._tenants.append(tenant)

    def remove_tenant(self, tenant):
        ''' 
        Remove the specified tenant from the list maintained by the controller.
        '''

        try: 
            self._tenants.remove(tenant)
            
        except:
            pass

    def get_tenants(self):

        '''
        Return the list of tenants known to the controller.
        '''

        return self._tenants

    def mark_dirty(self, dirty):
        '''
        Mark the tenants on this gateway dirty. This is done before
        we query keystone for the tenant list. Those that are not
        marked clean will be removed.
        '''

        for x in self.get_tenants():
            x.set_dirty(dirty)

    def sweep(self):
        ''' 
        Remove any tenants no longer known to OpenStack.
        '''

        for x in self.get_tenants():
            if x.is_dirty():
                self.remove_tenant(x)

    def get_next_VLAN(self):
        '''
        Allocate a VLAN for use by a tenant. 
        XXX need to do a better job than throwing an exception when
        the upper bound is reached. Also should be deallocating VLANs
        when a tenant goes away for reuse.
        '''

        self._vlan += 1
        if self._vlan > 4095:
            raise Exception
        return self._vlan

    def get_logical_switch(self, tenant, net):
        '''
        given a tenant and OpenStack network name, retrieve a logical
        switch.
        '''
        
        ret = None
        for x in self._logical_switches:
            if x.get_network_name() == net and x.get_tenant_name() == tenant:
                ret = x
                break
        return ret
    
    def create_logical_switch(self):
        '''
        Allocate a logical switch and add it to the list.
        '''
        ls = LogicalSwitch()
        self._logical_switches.append(ls)
        return ls

    def get_hypervisor_ip(self, hyper):

        '''
        Given a hypervisor object, return it's IPV4 address.
        '''

        ret = socket.gethostbyname(hyper["name"])
        if "127.0." in ret and ret[0:4] == "127.":
            ret = get_ip_address(vtepc_config.local_interface)
        return ret

    def create_ls_on_hypervisor(self, hypervisor, logical_switch):
        '''
        Connect to the specified hypervisor and create a Logical_Switch
        entry in its hardware_vtep database. 

        '''

        vtep = VTEPNode()
        vtep.set_ip(self.get_hypervisor_ip(hypervisor))
        lsname = logical_switch.get_name()
        vtep.vtep_ctl("--may-exist add-ls %s" % (lsname))
        vtep.vtep_ctl("set Logical_Switch %s tunnel_key=%s" 
                      % (lsname, logical_switch.get_segmentation_id()))

    def create_ls_on_gateway(self, gateway, logical_switch):
        '''
        Connexct to the specified VTEP gateway and create a logical switch 

        '''

        vtep = VTEPNode()
        print gateway
        vtep.set_ip(gateway.get_ip())
        lsname = logical_switch.get_name()
        vtep.vtep_ctl("--may-exist add-ls %s" % (lsname))
        vtep.vtep_ctl("set Logical_Switch %s tunnel_key=%s" 
                      % (lsname, logical_switch.get_segmentation_id()))
        
    def create_physical_locators(self, hypervisor, hypers, ls):

        '''
        on the specified hypervisor, create physical locators that
        terminate on a set of hypervisors (not including the source
        hypervisor), and bind to the specified logical switch.
        '''

        vtep = VTEPNode()
        vtep.set_ip(socket.gethostbyname(hypervisor["name"]))
        for x in hypers:
            lhost = socket.gethostbyname(x["name"])
            if "127.0." in lhost and lhost[0:4] == "127.":
                lhost = get_ip_address(vtepc_config.local_interface)
            print "On hypervisor %s and ls %s adding locator for %s" % \
                  (socket.gethostbyname(hypervisor["name"]),
                   ls.get_name(), lhost) 
            ret = vtep.vtep_ctl("add-mcast-remote %s unknown-dst %s" % 
                                (ls.get_name(), lhost))

    def create_physical_locators_to_gateway(self, gw, hypers, ls):

        '''
        on the specified gateway, create physical locators that
        terminate on a set of hypervisors (not including the source
        hypervisor), and bind to the specified logical switch.
        '''

        vtep = VTEPNode()
        for x in hypers:
            lhost = socket.gethostbyname(x["name"])
            if "127.0." in lhost and lhost[0:4] == "127.":
                lhost = get_ip_address(vtepc_config.local_interface)
            vtep.set_ip(lhost)
            ret = vtep.vtep_ctl("add-mcast-remote %s unknown-dst %s" % 
                                (ls.get_name(), gw.get_ip()))
        

    def create_physical_locators_on_gateway(self, gw, hypers, ls):

        '''
        on the specified gateway, create physical locators that
        terminate on a set of hypervisors (not including the source
        hypervisor), and bind to the specified logical switch.
        '''

        vtep = VTEPNode()
        vtep.set_ip(gw.get_ip())
        for x in hypers:
            lhost = socket.gethostbyname(x["name"])
            if "127.0." in lhost and lhost[0:4] == "127.":
                ip = get_ip_address(vtepc_config.local_interface)
            else:
                ip = socket.gethostbyname(x["name"])
            ret = vtep.vtep_ctl("add-mcast-remote %s unknown-dst %s" % 
                                (ls.get_name(), ip))

    def remove_physical_locators_from_gateway(self, gw, hypers, ls):

        '''
        on the specified gateway, remove physical locators and records
        that depend on them, for those that terminate on a set of hypervisors 
        (not including the source hypervisor), and are bound to the specified 
        logical switch.
        '''

        vtep = VTEPNode()
        vtep.set_ip(gw.get_ip())
        for x in hypers:
            lhost = socket.gethostbyname(x)
            if "127.0." in lhost and lhost[0:4] == "127.":
                ip = get_ip_address(vtepc_config.local_interface)
            else:
                ip = socket.gethostbyname(x)
            ret = vtep.vtep_ctl("del-mcast-remote %s unknown-dst %s" % 
                                (ls.get_name(), ip))

            locator = vtep.vtep_ctl("--columns=_uuid find Physical_Locator dst_ip=%s" % (ip))
            if locator == None or locator == "":
                continue
            try:
                uuid = locator.split(":")[1].strip()
                recs = vtep.vtep_ctl("--columns=_uuid find Ucast_Macs_Remote locator=%s" % (uuid)).splitlines()
                for y in recs:
                    uuid = y.split(":")[1].strip()
                    vtep.vtep_ctl("--if-exists destroy Ucast_Macs_Remote %s" %
                                   uuid)
            except:
                print "exception in remove_physical_locators_from_gateway"
                pass

    def process_mcast_local_tables(self):

        ''' 
        XXX deprecate this function after testing.
        '''

        for tenant in self._tenants:
            tname = tenant.get_name()
            nets = tenant.get_networks()
            for net in nets:
                netname = net["name"]
                ls = self.get_logical_switch(tname, netname)
                hypers = tenant.get_hypervisors()
                for hyper in hypers:
                    vtep = VTEPNode()
                    vtep.set_ip(self.get_hypervisor_ip(hyper))
                    ret = vtep.vtep_ctl("list-local-macs %s" % 
                                        (ls.get_name()))
                    ret = ret.splitlines()
                    tmp = []
                    for s in ret:
                        tmp.append(s.strip())
                    try:
                        for i in xrange(tmp.index("mcast-mac-local") + 1, len(tmp)):
                            mcast = tmp[i]
                            # for now (or forever) just handle unknown-dst
                            try:
                                x = mcast.index("unknown-dst")
                                x = mcast.index("->") + 2 
                                mcast = mcast[x:].split('/')
                                etype = mcast[0].strip()
                                ip = mcast[1].strip("'")
                                if ip ==  get_ip_address(vtepc_config.local_interface):
                                    # no tunnels to self
                                    continue

                                for x in hypers:
                                    if x == hyper:
                                        continue

                                    name = x["name"]
                                     
                                    if len(tenant.get_servers_on_hypervisor(name)):
                                        vtep.set_ip(socket.gethostbyname(name))
                                        ret = vtep.vtep_ctl(
                                            "add-mcast-remote %s unknown-dst %s" % 
                                            (ls.get_name(), ip))
                            except:
                                # unable to find unknown-dst
                                pass
                    except:
                        # unable to find mcast-mac-local
                        pass

    def create_vlan_binding(self, host, port, logical_switch, vlan):
        '''
        On the specified host, create the specified vlan binding for the port
        '''

        vtep = VTEPNode()
        vtep.set_ip(host)
        vtep.vtep_ctl("bind-ls %s %s %s %s" % \
                      ("br-int", port, vlan, logical_switch))

    def set_port_description(self, host, port, description):
        ''' 
        Write the description field for the specified port in the 
        physical_port table on the specified host.

        For details on how this is used, grep for callers.
        '''

        vtep = VTEPNode()
        vtep.set_ip(host)
        vtep.vtep_ctl("set physical_port %s description=%s" % 
                (port, description))

    def remove_gone_hypervisors(self, tenant, hypers):
        '''
        On all VTEP gateways in the cluster mapped to the specified
        tenant, remove physical locators corresponding to the list
        of hypervisors. This will result in tunnel teardown on the
        VTEP gateway.
        '''
        
        tname = tenant.get_name()
        gwmap = self.get_tenant_gateway_map(tname)
        for x in gwmap:
            gw = x["gateway"]
            nets = tenant.get_networks()
            for net in nets:
                netname = net["name"]
                ls = self.get_logical_switch(tname, netname)
                if ls != None:
                    self.remove_physical_locators_from_gateway(gw, hypers, ls)

    def update_logical_switches(self):
        ''' 
        Update logical switches on all nodes in the cluster.
        '''

        for tenant in self._tenants:
            tname = tenant.get_name()
            nets = tenant.get_networks()
            if not nets:
                continue
            for net in nets:
                netname = net["name"]
                ls = self.get_logical_switch(tname, netname)
                if ls == None:
                    # new logical switch. Create one then push out to nodes

                    ls = self.create_logical_switch()
                    ls.set_tenant_name(tname)
                    ls.set_network_name(netname)
                    ls.set_segmentation_id(net["segmentation_id"])

                # first update hypervisors

                hypers = tenant.get_hypervisors()
                gwmap = self.get_tenant_gateway_map(tname)

                for hyper in hypers:
                    if tenant.hyper_has_server_in_net(hyper, netname):
                        self.create_ls_on_hypervisor(hyper, ls)

                        # for every gateway in the cluster, tell
                        # the hypervisor about it with a binding

                        for x in gwmap:
                            gw = x["gateway"]
                            for dev in gw.get_devices():
                                if self.is_tenant_device(tname, dev.get_name()):
                                    self.create_vlan_binding(
                                       self.get_hypervisor_ip(hyper), 
                                       "patch-tun",
                                       ls.get_name(), 
                                       tenant.get_vlan())

                # now process the gateway(s)

                for x in gwmap:
                    gw = x["gateway"]
                    self.create_ls_on_gateway(gw, ls)
                    self.create_physical_locators_to_gateway(gw, hypers, ls)
                    self.create_physical_locators_on_gateway(gw, hypers, ls)
                    for dev in gw.get_devices():
                        if self.is_tenant_device(tname, dev.get_name()):
                            self.create_vlan_binding(gw.get_ip(), 
                                                     dev.get_port(),
                                                     ls.get_name(), 
                                                     tenant.get_vlan())

                            '''
                            sneakily use description to hold mac of dev
                            so the switch can inject the mac addr into
                            the learned mac table for tenant tunnels
                            '''

                            self.set_port_description(gw.get_ip(),
                                                      dev.get_port(),
                                                      dev.get_mac())

    def process_tenants(self):

        ''' 
        For each tenant, inventory the hypervisors and servers
        in the cluster corresponding to that tenant. Remove any 
        hypervisors that are gone since the last time we were 
        called.
        '''

        keystone = Keystone()
        nova = Nova()
        neutron = Neutron()

        tenant_list = keystone.get_tenant_list()
        for x in tenant_list:
            t = self.search(x["name"])
            if t == None:     
                tenant = Tenant()
                self.add_tenant(tenant)
            else:
                tenant = t
                tenant.set_dirty(False)
            tenant.set_id(x["id"])
            tenant.set_name(x["name"])
            tenant.set_servers(nova.get_server_list(tenant))

            hlist_cur = []
            t = tenant.get_hypervisors()
            if t:
                for y in t:
                    hlist_cur.append(y["name"])
           
            tenant.set_hypervisors(nova.get_hypervisor_list(tenant))
            tenant.set_networks(neutron.get_network_list(tenant))

            hlist = []
            t = tenant.get_hypervisors()
            for y in t:
                hlist.append(y["name"])

            hypers_gone = set(hlist_cur) - set(hlist)
            self.remove_gone_hypervisors(tenant, hypers_gone)
            for h in hlist:
                tenant.set_servers_on_hypervisor(h, nova.get_servers_on_hypervisor(tenant, h))

            '''
            y = tenant.get_servers()
            print "servers:"
            for a in y:
                print a["id"], a["name"]
                
            y = tenant.get_hypervisors()
            print "hypervisors:"
            for a in y:
                print a
                print "servers on %s" % (a["name"])
                z = tenant.get_servers_on_hypervisor(a["name"])
                for c in z:
                    print "id: %s name %s" % (c.id, c.name)

            y = tenant.get_networks()
            for a in y:
                print a["id"], a["type"], a["segmentation_id"]
            '''

    def add_remote_ucast_entry(self, host, logical_switch, ip, mac):

        '''
        On the specified host, and for the specified args, add an
        entry to the ucast remote table.
        '''
         
        vtep = VTEPNode()
        vtep.set_ip(host)
        vtep.vtep_ctl("add-ucast-remote %s %s %s" % \
                      (logical_switch.get_name(), mac, ip))

    def del_remote_ucast_entry(self, host, logical_switch, mac):
        '''
        On the specified host, remove the ucast remote table entry
        corresponding to the specified args.
        '''

        vtep = VTEPNode()
        vtep.set_ip(host)
        vtep.vtep_ctl("del-ucast-remote %s %s" % \
                      (logical_switch.get_name(), mac))

    def get_local_macs(self, host, logical_switch):

        '''
        On the specified host, retrieve the local macs for the 
        specified logical switch.
        '''

        vtep = VTEPNode()
        vtep.set_ip(host)
        lu = []
        try:
          ls = vtep.vtep_ctl("find logical_switch name=%s" % \
                           (logical_switch.get_name()))
          uuid = ls.splitlines()[0].split(':')[1].strip(' ')
          lu = vtep.vtep_ctl("find ucast-macs-local logical_switch=%s" % (uuid))
          lu = lu.splitlines()
        except:
          pass

        macs = []
        for i in lu:
            if i[0:3] == "MAC":
                mac = i[i.index('"'):].strip(' ').strip('"')
                macs.append(mac)
        return macs

    def update_remote_macs(self):

        '''
        On each VTEP gateways, populate the remote_ucast table with the 
        mac address to logical switch mappings for servers running in the 
        cluster on each hypervisor. Likewise, make sure every hypervisor 
        has remote_cast pointing back to the gateways. 

        This algorithm probably doesn't scale to massive clusters with
        lots of tenants and servers. Some possible ways to address this:

        - cache enough information about the remote_macs tables in the 
          cluster to avoid having to issue the network requests unless
          there is a change.

        - right now, this function is called each main loop iteration. 
          perhaps it can be called at a frequency which is the function
          of the size of the cluster, number of tenants, number of servers
          executing in the cluster.

        - instead of processing all tenants, use some heuristic (SLA?) to
          determine how often we need to service a given tenant.
        '''

        nova = Nova()
        for tenant in self._tenants:
            tname = tenant.get_name()
            nets = tenant.get_networks()
            if nets == None:
                continue
            for net in nets:
                netname = net["name"]
                ls = self.get_logical_switch(tname, netname)
                if ls == None:
                    print "update_remote_macs: no lswitch for %s %s" % \
                          (tname, netname)
                    continue
                # first update hypervisors

                hypers = tenant.get_hypervisors()
                gwmap = self.get_tenant_gateway_map(tname)

                for hyper in hypers:

                    local_ucasts = []

                    servers = nova.get_servers_on_hypervisor(tenant, 
                                                             hyper["name"])
                    for server in servers:
                   
                        try: 
                            addr = server.addresses[netname][0]
                        except:
                            continue

                        hyperip = self.get_hypervisor_ip(hyper)
                        local_ucasts.append(addr['OS-EXT-IPS-MAC:mac_addr'])                            
                        # for every gateway in the cluster, 
                        # add an entry to its remote ucast_table 

                       
                        for x in gwmap:
                            gw = x["gateway"]
                            gwip = gw.get_ip()
                            for y in local_ucasts:
                                self.add_remote_ucast_entry(gwip, 
                                                            ls,
                                                            hyperip,
                                                            y)

                            # read the gw locals for the logical switch

                            gw_locals = self.get_local_macs(gwip, ls)

                            # update the hypervisor so that it has an
                            # entry in its remote_ucast pointing to the
                            # gateway device
    
                            if gw_locals:
                                for y in gw_locals:
                                    self.add_remote_ucast_entry(hyperip, 
                                                                ls, 
                                                                gwip,
                                                                y)

def get_ip_address(ifname):

    '''
    Get the IP address of a specific named interface (e.g., "eth0")
    '''

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915,  # SIOCGIFADDR
        struct.pack('256s', ifname[:15])
    )[20:24])

def get_local_ip():
    """Return the actual ip of the local machine.

    This code figures out what source address would be used if some traffic
    were to be sent out to some well known address on the Internet. In this
    case, a Google DNS server is used, but the specific address does not
    matter much.  No traffic is actually sent.
    """
    csock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    csock.connect(('8.8.8.8', 80))
    (addr, _port) = csock.getsockname()
    csock.close()
    return addr

def call_prog(prog, args_list):

    '''
    Shell out to the specified program, passing the specified args.
    '''

    cmd = [prog] + args_list
    output = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
    if len(output) == 0 or output[0] == None:
        output = ""
    else:
        output = output[0].strip()
    return output

class Keystone():

    '''
    Class to abstract calls to the OpenStack Keystone service. Keystone
    is the service that manages tenants in OpenStack.
    '''

    def __init__(self):
        self.username = "admin"
        self.password = "password"
        self.local_ip = get_local_ip()
        self.tenant_list = None

    def connect(self, tenant_name, tenant_password):

        '''
        Return an object that represents a connection to Keystone.
        This object implements the Keystone API (version is specified
        by the URL).
        '''

        serviceurl = "http://%s:5000/v2.0/" % self.local_ip
        nt = None
        try:
            nt = keystone_client.Client(
                                        username=tenant_name,
                                        password=tenant_password,
                                        tenant_name=tenant_name, 
                                        auth_url=serviceurl
                                       )
        except:
            print "Keystone::connect fail"
            pass
        return nt

    def get_tenant_list(self):
        
        '''
        Get a list of id, name pairs representing the tenants known to
        keystone. Filter this list to include only those which are 
        configured by the controller to use VTEP gateway.
        '''
        
        if self.tenant_list == None:
            self.tenant_list = [] 
            nt = self.connect("admin", "password")
            if nt != None:
                x = nt.tenants.list()
                for y in x:
                    # only care about tenants configured to use gateway

                    if controller.search(y.name) != None:
                        self.tenant_list.append({"id": y.id, "name": y.name})
        return self.tenant_list

class Neutron():
    '''
    Class to abstract calls to the OpenStack Neutron service. Neutron
    is the service that manages networks in OpenStack.
    '''

    def __init__(self):
        self.username = "admin"
        self.password = "password"
        self.local_ip = get_local_ip()

    def connect(self, tenant_name, tenant_password):
        '''
        Return an object that represents a connection to Neutron. This 
        object implements the Neutron API (the API version is specified
        by the URL).
        '''

        serviceurl = "http://%s:5000/v2.0/" % self.local_ip
        nt = None
        try:
            nt = neutron_client.Client(username=tenant_name,
                                       password=tenant_password,
                                       tenant_name=tenant_name, 
                                       auth_url=serviceurl)
        except:
            print "Neutron::connect failed"
            pass

        return nt

    def get_network_list(self, tenant):
        '''
        Get a list of the networks for which a tenant has been 
        configured.
        '''
       
        ret = []
        tid = tenant.get_id()
        nt = self.connect(tenant.get_name(), tenant.get_password())
        if nt != None:
            try:
                nets = nt.list_networks()
                nets = nets["networks"]
                for x in nets:
                    if x["tenant_id"] != tid:
                        continue
                    ret.append({
                        "id": x["id"], 
                        "name": x["name"], 
                        "type": x["provider:network_type"], 
                        "tenant_id": x["tenant_id"], 
                        "segmentation_id": x["provider:segmentation_id"]
                    })
            except:
                pass
        return ret

class Nova():
    '''
    Class to abstract calls to the OpenStack Nova service. Nova
    is the service that manages compute in OpenStack, but it also
    implements APIs that report aspects of other services in the
    abstract, e.g., you can find out about networks via Nova and
    the API is agnostic to how networks are actually implemented.
    '''

    def __init__(self):
        self.username = "admin"
        self.password = "password"
        self.local_ip = get_local_ip()

    def connect(self, tenant_name, tenant_password):
        '''
        Return an object that represents a connection to Nova. This 
        object implements the Nova API (the API version is specified
        by the URL).
        '''

        serviceurl = "http://%s:5000/v2.0/" % self.local_ip
        nt = None
        try:
            nt = nova_client.Client(username=tenant_name, api_key=tenant_password,
                                    project_id=tenant_name, auth_url=serviceurl, 
                                    service_type="compute")
        except:
            print "Nova::connect fail"
            pass

        return nt


    def get_server_list(self, tenant):
        ''' 
        Get a list of the servers (regardless of state) associated
        with the specified tenant. A server is OpenStack's term for
        a VM running in a hypervisor.
        '''
       
        ret = []
        nt = self.connect(tenant.get_name(), tenant.get_password())
        if nt != None:
            try:
                list = nt.servers.list()
                for x in list:
                    ret.append({"id": x.id, "name": x.name})
            except:
                print "get_server_list exception", sys.exc_info()[0]
                pass
        return ret


    def get_hypervisor_list(self, tenant):

        '''
        Get a list of the hypervisors for a given tenant. Only return
        hypervisors which host ACTIVE tenant servers. List consists of 
        id, hostname tuples.
        '''

        ret = []
        nt = self.connect(tenant.get_name(), tenant.get_password())
        if nt != None:
            try:
                list = nt.hypervisors.list()
                for x in list:
                    name = x.hypervisor_hostname
                    servers = self.get_servers_on_hypervisor(tenant, name)
                    if len(servers):
                        ret.append({"id": x.id, "name": name})
            except:
                print "get_hypervisor_list exception" , sys.exc_info()[0]
                pass
        return ret

    def get_servers_on_hypervisor(self, tenant, name):

        '''
        Get a list of the servers for a given tenant, hypervisor pair. 
        Only return servers which are in the ACTIVE state.
        '''

        ret = []
        nt = self.connect(tenant.get_name(), tenant.get_password())
        if nt != None:
            try:
                sret = nt.hypervisors.search(name, True)
                print sret[0]
                for x in sret[0].servers:
                    server = nt.servers.get(x["uuid"])
                    if server.status == "ACTIVE":
                        print "found ACTIVE server on hypervisor %s" % (name)
                        ret.append(server)
            except:
                pass
        return ret

    def set_password(self, password):
        self.password = password

    def set_username(self, username):
        self.username = username

class VTEPNode():

    '''
    Class to abstract a VTEP at some location in the cluster.
    In general, this class is instantiated once for each 
    invocation. Setters are called to specify the host and port
    of the VTEP to which a command or commands are to be sent.
    Then, commands on the vtep are invoked by calling vtep_ctl
    member function.
    '''

    def __init__(self):
        self._ip = None
        self._port = 6632

    def vtep_ctl(self, args):

        '''
        Issue a call to the configured VTEP. The commands is args[0],
        the rest of the vector contains the remaining arguments.
        '''
        
        url = self.get_db_arg()
        return call_prog("vtep-ctl", ["-vconsole:off", "--timeout=10", url] + args.split())

    def set_ip(self, ip):
        self._ip = ip

    def get_ip(self):
        return self._ip

    def set_port(self, port):
        self._port = ip

    def get_port(self):
        return self._port

    def get_db_arg(self):
        '''
        Construct the "db" argument to vtep-ctl based on configured
        IP address and port, and return the resulting argument.
        '''
 
        return "--db=tcp:%s:%d" % (self.get_ip(), self.get_port())

controller = GatewayController()

def load_config():
    for x in vtepc_config.gateways:
      gateway = Gateway()
      gateway.set_ip(x["ip"])
      controller.add_gateway(gateway)

    for x in vtepc_config.tenants:
      tenant = Tenant()
      tenant.set_name(x["name"])
      tenant.set_password(x["password"])
      tenant.set_vlan(controller.get_next_VLAN())
      controller.add_tenant(tenant)
    
    for x in vtepc_config.devices:
      device = Device()
      device.set_name(x["name"])
      device.set_description(x["description"])
      device.set_mac(x["mac"])
      device.set_port(x["port"])
      device.set_gateway(x["gateway"])
      gateway = controller.get_gateway(x["gateway"])
      gateway.add_device(device)
      controller.add_device(device)

    for x in vtepc_config.tenantdevmap:
      tenant = controller.get_tenant(x["tenant"])
      for y in x["devices"]:
        device = controller.get_device(y)
        controller.map_device_to_tenant(tenant.get_name(), device.get_name())
        gateway = controller.get_gateway(device.get_gateway())
        controller.map_gateway_to_tenant(tenant.get_name(), gateway)
      
     
def main():

    # daemonize

    parser = argparse.ArgumentParser()
    ovs.vlog.add_args(parser)
    ovs.daemon.add_args(parser)
    args = parser.parse_args()
    ovs.vlog.handle_args(args)
    ovs.daemon.handle_args(args)
    ovs.daemon.daemonize()

    load_config()

    '''
    The main loop of controller. Loop forever, processing
    hypervisors, tenants, and servers as they come and go.
    '''
    
    while True:
        controller.mark_dirty(True) 
        controller.process_tenants()
        controller.update_logical_switches()
        controller.update_remote_macs()
        #controller.process_mcast_local_tables()
        controller.sweep() 
        time.sleep(10)

if __name__ == '__main__':
    main()
